use std::error::Error;
use std::fs;
use std::path::Path;

use clap::{ArgGroup, Parser};
use common::{Day, Part};

#[derive(Parser, Debug)]
#[command(about)]
#[command(group(
ArgGroup::new("input")
.required(true)
.args(["input_file", "input_str"])
))]
struct Args {
    #[arg(short, long)]
    day: Day,

    #[arg(short, long)]
    part: Part,

    #[arg(short = 'f', long)]
    input_file: Option<String>,

    input_str: Option<String>,
}

fn main() -> Result<(), Box<dyn Error>> {
    let cli = Args::parse();

    let input_str = if let Some(inp) = cli.input_file.as_deref() {
        fs::read_to_string(Path::new(inp))?
    } else {
        cli.input_str.unwrap()
    };

    match (cli.day, cli.part) {
        (day @ Day::One, part @ Part::One) => {
            println!("{}", day_01::run_day_and_part(day, part, input_str));
            Ok(())
        }
        (day @ Day::One, part @ Part::Two) => {
            println!("{}", day_01::run_day_and_part(day, part, input_str));
            Ok(())
        }
        (day @ Day::Two, part @ Part::One) => {
            println!(
                "{}",
                day_02::run_day_and_part(day, part, input_str.as_str())
            );
            Ok(())
        }
        (day @ Day::Two, part @ Part::Two) => {
            println!(
                "{}",
                day_02::run_day_and_part(day, part, input_str.as_str())
            );
            Ok(())
        }
        (day @ Day::Three, part @ Part::One) => {
            println!(
                "{}",
                day_03::run_day_and_part(day, part, input_str.as_str())
            );
            Ok(())
        }
        (day @ Day::Three, part @ Part::Two) => {
            println!(
                "{}",
                day_03::run_day_and_part(day, part, input_str.as_str())
            );
            Ok(())
        }

        (_, _) => {
            unimplemented!()
        }
    }
}
