use chunk_iter::ChunkIter;
use common::{Day, Part};
use phf::phf_map;
use std::collections::HashSet;
use intersection::hash_set;
use itertools::{Itertools, kmerge};

static VALUES: phf::Map<char, usize> = phf_map! {
    'a' => 1,
    'b' => 2,
    'c' => 3,
    'd' => 4,
    'e' => 5,
    'f' => 6,
    'g' => 7,
    'h' => 8,
    'i' => 9,
    'j' => 10,
    'k' => 11,
    'l' => 12,
    'm' => 13,
    'n' => 14,
    'o' => 15,
    'p' => 16,
    'q' => 17,
    'r' => 18,
    's' => 19,
    't' => 20,
    'u' => 21,
    'v' => 22,
    'w' => 23,
    'x' => 24,
    'y' => 25,
    'z' => 26,
    'A' => 27,
    'B' => 28,
    'C' => 29,
    'D' => 30,
    'E' => 31,
    'F' => 32,
    'G' => 33,
    'H' => 34,
    'I' => 35,
    'J' => 36,
    'K' => 37,
    'L' => 38,
    'M' => 39,
    'N' => 40,
    'O' => 41,
    'P' => 42,
    'Q' => 43,
    'R' => 44,
    'S' => 45,
    'T' => 46,
    'U' => 47,
    'V' => 48,
    'W' => 49,
    'X' => 50,
    'Y' => 51,
    'Z' => 52,
};

pub fn run_day_and_part(day: Day, part: Part, inp: &str) -> usize {
    match (day, part) {
        (Day::Three, Part::One) => part01_parse(inp).iter().sum(),
        (Day::Three, Part::Two) => part02_parse(inp).iter().sum(),
        (_, _) => unimplemented!(),
    }
}

fn part01_parse(inp: &str) -> Vec<usize> {
    let inp = inp.split_whitespace();
    inp.map(|row: &str| row.split_at(row.len() / 2))
        .map(|(left, right): (&str, &str)| {
            (
                HashSet::from_iter(left.chars()),
                HashSet::from_iter(right.chars()),
            )
        })
        .map(|(left, right): (HashSet<char>, HashSet<char>)| {
            *left.intersection(&right).next().unwrap()
        })
        .map(|i: char| VALUES[&i])
        .collect::<Vec<usize>>()
}

fn part02_parse(inp: &str) -> Vec<usize> {
    let inp = inp.split_whitespace();
    ChunkIter::chunks::<3>(inp.map(|it| HashSet::from_iter(it.chars())))
        .map(|it| match it {
            [first, second, third] => dbg!((first, second, third)),
        })
        .map::<char, _>(
            |(first, second, third): (HashSet<char>, HashSet<char>, HashSet<char>)| {
                *hash_set::intersection(vec![first,second,third]).iter().next().unwrap()
            },
        )
        .map(|it| dbg!(it))
        .map(|i: char| VALUES[&i])
        .collect()
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn part_01_example_works() {
        let input = r#"vJrwpWtwJgWrhcsFMMfFFhFp
jqHRNqRjqzjGDLGLrsFMfFZSrLrFZsSL
PmmdzqPrVvPwwTWBwg
wMqvLMZHhHMvwLHjbvcjnnSBnvTQFn
ttgJtRGJQctTZtZT
CrZsJsPPZsGzwwsLwLmpwMDw"#;
        assert_eq!(part01_parse(input), vec![16, 38, 42, 22, 20, 19]);
    }

    #[test]
    fn part_02_example_works() {
        let input = r#"vJrwpWtwJgWrhcsFMMfFFhFp
jqHRNqRjqzjGDLGLrsFMfFZSrLrFZsSL
PmmdzqPrVvPwwTWBwg
wMqvLMZHhHMvwLHjbvcjnnSBnvTQFn
ttgJtRGJQctTZtZT
CrZsJsPPZsGzwwsLwLmpwMDw"#;
        assert_eq!(part02_parse(input), vec![18, 52])
    }
}
